#!/usr/bin/env python3

import re
from typing import Any

from ..extension import Extension
from ..tree import SourceNode, SourceIndex

"""The extension for generating breadcrumb informatioon."""


class Breadcrumbs(Extension):
    LINK = re.compile(r'^\[\[(.+)(?:\|.+)?\]\]')

    def __init__(self, config: dict[str, Any]):
        """
        Initialize the Breadcrumbs extension.

        Args:
            config (dict[str, Any]): Configuration variables for this extension.
        """
        super(Breadcrumbs, self).__init__(config)
        self._root_nodes: list[SourceNode] = []
        self._raw_targets: list[tuple[SourceNode, str]] = []
        self._parents: dict[SourceNode, SourceNode] = {}

    def loaded_metadata(self, ctxt: Any, node: SourceNode) -> None:
        """
        Called after the metadata for a node has been loaded.  Should be overridden by derived classes.

        Args:
            ctxt (Context): Context for the operation.
            node (SourceNode): Node that's just had its metadata loaded.
        """
        if node.publish:
            target_str = node.metadata.get("parent", None)
            if target_str:
                m = self.LINK.match(str(target_str))
                if m:
                    target_str = m.group(1)
                if target_str == '.':
                    self._root_nodes.append(node)
                else:
                    self._raw_targets.append((node, target_str))

    def indexed(self, ctxt: Any, index: SourceIndex) -> None:
        """
        Called after the index for the nodes has been generated.

        Args:
            ctxt (Context): Context for the operation.
            index (SourceIndex): The index to all nodes.
        """
        for node, target in self._raw_targets:
            target_node, _ = index.lookup(node, target)
            if target_node:
                self._parents[node] = target_node

    def supply_template_vars(self, ctxt: Any, node: SourceNode) -> dict[str, Any] | None:
        """
        Called to supply additional template variables to the template to be rendered.

        Args:
            ctxt (Context): Context for the operation.
            node (SourceNode): The node currently being processed for template rendering.

        Returns:
            dict[str, Any]: If not ``None``, contains additional template variables to be added before
                the template is rendered.
        """
        breadcrumbs: list[dict[str, str]] = [{"name": "." if node in self._root_nodes else node.page_title,
                                              "link": ""}]
        cur_node = node
        while cur_node not in self._root_nodes:
            cur_node = self._parents.get(cur_node, None)
            if not cur_node:
                return None
            breadcrumbs.insert(0, {"name": "." if cur_node in self._root_nodes else cur_node.page_title,
                                   "link": self.link_target(ctxt, node, cur_node)})
        return {"breadcrumbs": breadcrumbs}
